import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

class MyFancyDialog extends StatelessWidget {
  final String title;
  final String body;
  final IconData icon;
  final Color color;
  final Color iconColor;
  final double radius;
  final double iconSize;
  final double radiusSmall;
  final double buttonHeight;
  final List<String> actionsInRow;
  final List<String> actionsInCol;
  final List<String> inputs;
  final List<String> numberInputs;
  final List<String> multiLineInputs;
  final List<TextEditingController> inputControllers;
  final List<FocusNode> inputFNS;

  MyFancyDialog(
      {this.buttonHeight = 50,
      this.radiusSmall = 6,
      this.radius = 25,
      this.iconSize = 50,
      this.iconColor = Colors.white,
      this.actionsInCol = const ["Cancel"],
      this.actionsInRow = const ["Confirm"],
      this.title = "",
      this.body = "",
      this.icon,
      this.inputs = const [],
      this.numberInputs = const [],
      this.multiLineInputs = const [],
      this.color = const Color(0xfffad82f)})
      : inputControllers = (inputs + numberInputs + multiLineInputs).map((e) => TextEditingController()).toList(),
        inputFNS = (inputs + numberInputs + multiLineInputs).map((e) => FocusNode()).toList();

  @override
  Widget build(BuildContext context) {
    double width = MediaQuery.of(context).size.width;
    double height = MediaQuery.of(context).size.height;

    return Dialog(
      backgroundColor: Colors.transparent,
      shape: RoundedRectangleBorder(
          borderRadius: BorderRadius.only(topLeft: Radius.circular(radius), topRight: Radius.circular(radius))),
      child: Container(
        decoration: BoxDecoration(
            color: Colors.transparent,
            borderRadius: BorderRadius.only(topLeft: Radius.circular(radius), topRight: Radius.circular(radius))),
        width: width * 0.8,
//        height: height * 0.5,
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: [
            Container(
              decoration: BoxDecoration(
                color: color,
                borderRadius: BorderRadius.only(topLeft: Radius.circular(radius), topRight: Radius.circular(radius)),
              ),
              padding: EdgeInsets.symmetric(horizontal: 4, vertical: 4),
              child: Row(
                children: [
                  Icon(icon ?? Icons.error, size: iconSize, color: iconColor),
                  SizedBox(width: 10),
                  Expanded(
                      child: Text(title ?? "Alert",
                          style: TextStyle(fontSize: 20, color: Colors.white, fontWeight: FontWeight.bold)))
                ],
              ),
            ),
            Divider(color: color.withOpacity(0.8), height: 1),
            Container(
              constraints: BoxConstraints(minHeight: height / 5, minWidth: double.infinity, maxHeight: height * 0.3),
              decoration: BoxDecoration(
                color: Colors.white,
                borderRadius:
                    BorderRadius.only(bottomLeft: Radius.circular(radiusSmall), bottomRight: Radius.circular(radiusSmall)),
              ),
              padding: EdgeInsets.only(left: 4, right: 4, top: 14, bottom: 10),
              width: width,
              child: SingleChildScrollView(
                child: Column(
                  mainAxisSize: MainAxisSize.min,
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    Padding(
                      padding: const EdgeInsets.all(2.0),
                      child: Center(
                        child: Text(
                          body,
                          textAlign: TextAlign.center,
                          style: TextStyle(fontSize: 16),
                        ),
                      ),
                    ),
                    SizedBox(
                      height: body.length > 0 ? 10 : 0,
                    ),
                    Column(
                      children: inputs
                          .map(
                            (e) => Padding(
                              padding: const EdgeInsets.all(5.0),
                              child: CupertinoTextField(
                                placeholder: e,
                                onSubmitted: (v) {
                                  if (inputs.indexOf(e) == inputs.length - 1) {
                                  } else {
                                    inputFNS[inputs.indexOf(e) + 1].requestFocus();
                                  }
                                },
                                focusNode: inputFNS[(inputs + numberInputs + multiLineInputs).indexOf(e)],
                                controller: inputControllers[(inputs + numberInputs + multiLineInputs).indexOf(e)],
                              ),
                            ),
                          )
                          .toList(),
                    ),
                    Column(
                      children: numberInputs
                          .map(
                            (e) => Padding(
                              padding: const EdgeInsets.all(5.0),
                              child: CupertinoTextField(
                                keyboardType: TextInputType.numberWithOptions(signed: true, decimal: true),
                                placeholder: e,
                                onSubmitted: (v) {
                                  if (numberInputs.indexOf(e) == numberInputs.length - 1) {
                                  } else {
                                    inputFNS[(inputs + numberInputs + multiLineInputs).indexOf(e) + 1].requestFocus();
                                  }
                                },
                                focusNode: inputFNS[(inputs + numberInputs + multiLineInputs).indexOf(e)],
                                controller: inputControllers[(inputs + numberInputs + multiLineInputs).indexOf(e)],
                              ),
                            ),
                          )
                          .toList(),
                    ),
                    Column(
                      children: multiLineInputs
                          .map(
                            (e) => Padding(
                              padding: const EdgeInsets.all(5.0),
                              child: CupertinoTextField(
                                maxLines: null,
                                placeholder: e,
                                onSubmitted: (v) {
                                  if (multiLineInputs.indexOf(e) == multiLineInputs.length - 1) {
                                  } else {
                                    inputFNS[(inputs + numberInputs + multiLineInputs).indexOf(e) + 1].requestFocus();
                                  }
                                },
                                focusNode: inputFNS[(inputs + numberInputs + multiLineInputs).indexOf(e)],
                                controller: inputControllers[(inputs + numberInputs + multiLineInputs).indexOf(e)],
                              ),
                            ),
                          )
                          .toList(),
                    )
                  ],
                ),
              ),
            ),
            Row(
              children: actionsInRow.map((e) {
                bool isFirst = actionsInRow.indexOf(e) == 0;
                bool isLast = actionsInRow.indexOf(e) == actionsInRow.length - 1;
                return Expanded(
                    child: FlatButton(
                  padding: EdgeInsets.all(0),
                  height: buttonHeight,
                  shape: RoundedRectangleBorder(
                      borderRadius: BorderRadius.only(
                          topLeft: Radius.circular(isFirst ? radiusSmall : 0),
                          bottomLeft: Radius.circular(isFirst ? radiusSmall : 0),
                          bottomRight: Radius.circular(isLast ? radiusSmall : 0),
                          topRight: Radius.circular(isLast ? radiusSmall : 0))),
                  onPressed: () => Navigator.pop(
                      context, e == "Cancel" ? null : DialogResult(e, inputControllers.map((e) => e.text).toList())),
                  child: Text(e, style: TextStyle(color: Colors.blueAccent, fontWeight: FontWeight.bold)),
                  color: Colors.white,
                ));
              }).toList(),
            ),
            Column(
              children: actionsInCol.map((e) {
                return Column(
                  children: [
                    SizedBox(
                      width: width,
                      height: buttonHeight,
                      child: FlatButton(
                        padding: EdgeInsets.all(0),
                        height: buttonHeight,
                        shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(radiusSmall)),
                        onPressed: () => Navigator.pop(
                            context, e == "Cancel" ? null : DialogResult(e, inputControllers.map((e) => e.text).toList())),
                        child: Text(e, style: TextStyle(color: Colors.blueAccent, fontWeight: FontWeight.bold)),
                        color: Colors.white,
                      ),
                    ),
                  ],
                );
              }).toList(),
            ),
          ],
        ),
      ),
    );
  }
}

class DialogResult {
  String action;
  List<String> inputs;

  DialogResult(this.action, this.inputs);

  @override
  String toString() {
    // TODO: implement toString
    return "$action ${inputs.join("-")}";
  }
}
