import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_picker/flutter_picker.dart';

class MySimpleListPickerDialog extends StatefulWidget {
  final List<SimpleListItem> items;
  final bool multi;
  final bool hasOnlineSearch;
  final bool isWheel;
  final bool canAddItem;
  final Widget title;
  final double height;
  final bool selectOnTap;
  final Widget divider;
  final EdgeInsetsGeometry insetPadding;
  final EdgeInsetsGeometry contentPadding;

  const MySimpleListPickerDialog(
      {Key key,
      this.canAddItem = false,
      this.items,
      this.multi = false,
      this.hasOnlineSearch = false,
      this.isWheel = false,
      this.height = 220,
      this.divider,
      this.selectOnTap = true,
      this.insetPadding = const EdgeInsets.symmetric(horizontal: 40, vertical: 10),
      this.contentPadding = const EdgeInsets.symmetric(vertical: 2, horizontal: 5),
      this.title})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return _MySimpleListPickerDialogState();
  }
}

class _MySimpleListPickerDialogState extends State<MySimpleListPickerDialog> {
  TextEditingController searchC = TextEditingController();
  String searched = "";
  List<int> selectedIndexes = [];
  List<int> selectedIndexesOnline = [];
  List<SimpleListItem> onlineResultItems = [];
  Function(BuildContext context, String searchedStr, Function(List<SimpleListItem> items) handleData) onlineSearchFunction;

  @override
  void initState() {
    searchC.text = "";
    if(widget.isWheel){
      selectedIndexes=[0];
    }
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return AlertDialog(
      shape: RoundedRectangleBorder(
        borderRadius: BorderRadius.circular(5),
      ),
      elevation: 10.0,
      contentPadding: widget.contentPadding,
      insetPadding: widget.insetPadding,
      backgroundColor: Colors.white,
      content: dialogContent(context),
    );
  }

  dialogContent(BuildContext context) {
    List<SimpleListItem> list;
    List<PickerItem> wheelList;
    Picker picker;
    if (widget.isWheel) {
      wheelList = widget.items.map((e) => PickerItem(value: e.val, text: e.renderItem)).toList();
      picker = Picker(
          adapter: PickerDataAdapter(data: wheelList),
          hideHeader: true,
          onSelect: (picker, i, iList) {
            setState(() {
              selectedIndexes = picker.selecteds;
            });
          });
    } else {
      list = (widget.items).where((element) => element.strVal.toLowerCase().contains(searched.toLowerCase())).toList();
    }

    return GestureDetector(
        onTap: () {
          FocusScope.of(context).requestFocus(FocusNode());
        },
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: <Widget>[
            widget.isWheel
                ? Container()
                : Container(
                    height: 70,
                    width: MediaQuery.of(context).size.width,
                    child: TextField(
                      decoration: InputDecoration(prefixIcon: Icon(Icons.search)),
                      onChanged: (v) {
                        setState(() {
                          searched = v;
                          if (widget.hasOnlineSearch &&
                              v.length > 2 &&
                              !list.any((element) => element.strVal.toLowerCase().contains(v.toLowerCase()))) {
                            onlineSearchFunction(context, v, (items) {
                              setState(() {
                                onlineResultItems = items;
                              });
                            });
                          }
                        });
                      },
                      controller: searchC,
                    ),
                  ),
            widget.title ?? Container(),
            widget.isWheel
                ? Container(
                    child: picker.makePicker(),
                  )
                : Container(
                    height: widget.height,
                    width: MediaQuery.of(context).size.width,
                    child: list.length < 1
                        ? onlineResultItems.length > 0
                            ? ListView.builder(
                                itemCount: onlineResultItems.length,
                                itemBuilder: (context, i) => widget.selectOnTap
                                    ? Column(
                                        children: <Widget>[
                                          Container(
                                            child: Material(
                                              child: InkWell(
                                                onTap: () {
                                                  Navigator.pop(context, [list[i].val]);
                                                },
                                                child: Container(
                                                  child: list[i].renderItem,
                                                ),
                                              ),
                                              color: Colors.transparent,
                                            ),
                                            color: list[i].bgColor,
                                          ),
                                          widget.divider ?? Container()
                                        ],
                                      )
                                    : Column(
                                        children: <Widget>[
                                          CheckboxListTile(
                                            dense: true,
                                            title: onlineResultItems[i].renderItem,
                                            value:
                                                selectedIndexesOnline.contains(onlineResultItems.indexOf(onlineResultItems[i])),
                                            onChanged: (v) {
                                              int index = onlineResultItems.indexOf(onlineResultItems[i]);
                                              print(v);
                                              print(index);
                                              print(selectedIndexesOnline);
                                              if (widget.multi) {
                                                if (selectedIndexesOnline.contains(index)) {
                                                  setState(() {
                                                    selectedIndexesOnline.remove(index);
                                                  });
                                                } else {
                                                  setState(() {
                                                    selectedIndexesOnline.add(index);
                                                  });
                                                }
                                              } else {
                                                if (selectedIndexesOnline.contains(index)) {
                                                  setState(() {
                                                    selectedIndexesOnline.remove(index);
                                                  });
                                                } else {
                                                  setState(() {
                                                    selectedIndexesOnline = [index];
                                                  });
                                                }
                                              }
                                            },
                                          ),
                                          widget.divider ?? Container()
                                        ],
                                      ))
                            : Center(child: Text("No item!"))
                        : ListView.builder(
                            itemCount: list.length,
                            itemBuilder: (context, i) => widget.selectOnTap
                                ? Column(
                                    children: <Widget>[
                                      Container(
                                        child: Material(
                                          child: InkWell(
                                            onTap: () {
                                              Navigator.pop(context, [list[i].val]);
                                            },
                                            child: Container(
                                              child: list[i].renderItem,
                                            ),
                                          ),
                                          color: Colors.transparent,
                                        ),
                                        color: list[i].bgColor,
                                      ),
                                      widget.divider ?? Container()
                                    ],
                                  )
                                : Column(
                                    children: <Widget>[
                                      CheckboxListTile(
                                        dense: true,
                                        title: list[i].renderItem,
                                        value: selectedIndexes.contains(widget.items.indexOf(list[i])),
                                        onChanged: (v) {
                                          int index = widget.items.indexOf(list[i]);
                                          if (widget.multi) {
                                            if (selectedIndexes.contains(index)) {
                                              setState(() {
                                                selectedIndexes.remove(index);
                                              });
                                            } else {
                                              setState(() {
                                                selectedIndexes.add(index);
                                              });
                                            }
                                          } else {
                                            if (selectedIndexes.contains(index)) {
                                              setState(() {
                                                selectedIndexes.remove(index);
                                              });
                                            } else {
                                              setState(() {
                                                selectedIndexes = [index];
                                              });
                                            }
                                          }
                                        },
                                      ),
                                      widget.divider ?? Container()
                                    ],
                                  )),
                  ),
            widget.canAddItem
                ? Align(
                    alignment: Alignment.centerRight,
                    child: IconButton(
                      icon: Icon(Icons.add_circle),
                      onPressed: () {
                        Navigator.pop(context, "ADD_NEW");
                      },
                    ),
                  )
                : Container(),
            ButtonBar(
              alignment: MainAxisAlignment.center,
              children: <Widget>[
                RaisedButton(
                  color: Colors.grey,
                  child: Container(
                      width: MediaQuery.of(context).size.width * 0.3,
                      child: Center(
                          child: Text(
                        "Cancel",
                        style: TextStyle(color: Colors.white),
                      ))),
                  onPressed: () {
                    Navigator.pop(context);
                  },
                ),
                widget.selectOnTap
                    ? Container()
                    : RaisedButton(
                        color: Colors.green,
                        child: Container(
                            width: MediaQuery.of(context).size.width * 0.3,
                            child: Center(
                                child: Text(
                              "Confirm",
                              style: TextStyle(color: Colors.white),
                            ))),
                        onPressed: () {
                          List results = selectedIndexes.map((e) => widget.items[e].val).toList() +
                              selectedIndexesOnline.map((e) => onlineResultItems[e].val).toList();
                          Navigator.pop(context, results);
                        },
                      ),
              ],
            )
          ],
        ));
  }
}

class SimpleListItem {
  Widget renderItem;
  String strVal;
  Color bgColor;
  dynamic val;

  SimpleListItem({this.val, this.renderItem, this.strVal, this.bgColor = Colors.white});
}
